

set -ex



pushd tests
pushd pyboost
./pyboost_test.sh
popd
popd
test -f $PREFIX/include/boost/python.hpp
test -d $PREFIX/include/boost/python
test -f $PREFIX/lib/libboost_python310.a
test -f $PREFIX/lib/libboost_python310${SHLIB_EXT}
test -f $PREFIX/lib/libboost_numpy310.a
test -f $PREFIX/lib/libboost_numpy310${SHLIB_EXT}
exit 0
